//
//  BigNumber.m
//  e
//
//  Created by Clemens Wagner on 03.08.14.
//  Copyright (c) 2014 Clemens Wagner. All rights reserved.
//

#import "BigNumber.h"
#import "NSString+BigNumber.h"
#import <GMP/gmp.h>

@implementation BigNumber {
    mpf_t _value;
}

+ (void)setPrecision:(NSUInteger)inPrecision {
    mpf_set_default_prec(inPrecision);
}

+ (instancetype)bigNumberWithInteger:(NSInteger)inValue {
    return [[BigNumber alloc] initWithInteger:inValue];
}

+ (instancetype)bigNumberWithString:(NSString *)inValue base:(int)inBase {
    return [[BigNumber alloc] initWithString:inValue base:inBase];
}

- (instancetype)init {
    self = [super init];
    if (self) {
        mpf_init_set_si(_value, 0);
    }
    return self;
}

- (instancetype)initWithInteger:(NSInteger)inValue {
    self = [super init];
    if (self) {
        mpf_init_set_si(_value, inValue);
    }
    return self;
}

- (instancetype)initWithString:(NSString *)inValue base:(int)inBase {
    self = [super init];
    if (self) {
        mpf_init_set_str(_value, [inValue cStringUsingEncoding:NSASCIIStringEncoding], inBase);
    }
    return self;
}

- (void)dealloc {
    mpf_clear(_value);
}

- (instancetype)bigNumberByAddingBigNumber:(BigNumber *)inValue {
    BigNumber *theResult = [BigNumber new];
    
    mpf_add(theResult->_value, _value, inValue->_value);
    return theResult;
}

- (instancetype)bigNumberByDividingUnsignedInteger:(NSUInteger)inValue {
    BigNumber *theResult = [BigNumber new];

    mpf_div_ui(theResult->_value, _value, inValue);
    return theResult;
}

- (NSString *)description {
    mp_exp_t theExponent;
    char *theData = mpf_get_str(NULL, &theExponent, 10, 0, _value);
    NSUInteger theLength = strlen(theData);
    NSMutableString *theResult = [NSMutableString new];
    char *theDigits;

    if(*theData == '-') {
        [theResult appendString:@"-"];
        theDigits = theData + 1;
    }
    else {
        theDigits = theData;
    }
    if(theExponent > 0) {
        [theResult appendFormat:@"%s", theDigits];
        if(theExponent > theLength) {
            [theResult appendString:[NSString zeroStringWithLength:theExponent - theLength]];
        }
        else {
            [theResult insertString:@"." atIndex:theExponent + (theDigits - theData)];
        }
    }
    else {
        [theResult appendString:@"0."];
        [theResult appendString:[NSString zeroStringWithLength:-theExponent]];
        [theResult appendFormat:@"%s", theDigits];
    }
    free(theData);
    return [theResult copy];
}

+ (BigNumber *)eulerNumberWithSteps:(NSUInteger)inSteps {
    BigNumber *theSum = [BigNumber bigNumberWithInteger:1];
    BigNumber *theComponent = [BigNumber bigNumberWithInteger:1];
    
    for(NSUInteger i = 1; i < inSteps; ++i) {
        theComponent = [theComponent bigNumberByDividingUnsignedInteger:i];
        
        theSum = [theSum bigNumberByAddingBigNumber:theComponent];
    }
    return theSum;
}

@end
