#!/bin/sh

export CPSUFFIX=a
export MINIMAL_TARGET=6.0
export CFLAGS="-O2 -miphoneos-version-min=$MINIMAL_TARGET"

download_and_extract() {
	if [ ! -f $1 ]
	then
		curl -L $3 > $1
	fi
	if [ ! -d $2 ]
	then
		tar xf $1
	fi
}

setup_environment() {
	sdk=$1
	arch=$2

	if [ $sdk == 'iphoneos' ]
	then
		export CPP="xcrun --sdk $sdk cpp -D__arm__"
	else
		export CPP="xcrun --sdk $sdk cpp"
	fi

	export CXX="xcrun --sdk $sdk clang++ -arch $arch"
 	export CC="xcrun --sdk $sdk clang -arch $arch"
 
	export LD="xcrun --sdk $sdk ld"
	export AR="xcrun --sdk $sdk ar"
	export AS="xcrun --sdk $sdk as"
	export NM="xcrun --sdk $sdk nm"
	export RANLIB="xcrun --sdk $sdk ranlib"
	
	export CPPFLAGS=$CFLAGS
	export CXXFLAGS=$CFLAGS
}

configure_and_make() {
	outdir="$BUILD_DIR/$2"
	lib="$outdir/$LIBNAME"
	if [ ! -f $lib ]
	then
		echo "create $LIBNAME for $1/$2"
		pushd $SRC_DIR
		rm -rf "$outdir"
		mkdir -p "$outdir"
		make -s clean 2> /dev/null
		make -s distclean 2> /dev/null
		setup_environment $*
		if [ -z "$3" ]
		then
			host_flag=""
		else
			host_flag="--host=$3"
		fi
		echo ./configure $host_flag $CONFIGURE_FLAGS
		./configure $host_flag $CONFIGURE_FLAGS || exit;
		pushd $MAKE_DIR
		make -j4
		popd
		find . -name "$LIBNAME" -exec cp {} $lib \;
		popd
	fi
}

make_archs() {
	sdk=$1
	host=$2
	shift
	shift
	for arch in $*
	do
		configure_and_make $sdk $arch "$host"
	done
}

make_bundle() {
	framework="$1.framework"
	rm -rf $framework
	mkdir -p "$framework/Versions/A/Headers"
	mkdir -p "$framework/Versions/A/Resources"
	pushd $framework
	ln -s "Versions/A/$1" $1
	ln -s "Versions/A/" Current
	ln -s "Versions/A/Headers" Headers
	ln -s "Versions/A/Resources" Resources
	cat > "Versions/A/Resources/Info.plist" << EOF
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
	<key>CFBundleDevelopmentRegion</key>
	<string>English</string>
	<key>CFBundleExecutable</key>
	<string>$1</string>
	<key>CFBundleIconFile</key>
	<string></string>
	<key>CFBundleIdentifier</key>
	<string>de.cocoaneheads.$1</string>
	<key>CFBundleInfoDictionaryVersion</key>
	<string>6.0</string>
	<key>CFBundleName</key>
	<string>$1</string>
	<key>CFBundlePackageType</key>
	<string>FMWK</string>
	<key>CFBundleShortVersionString</key>
	<string>1.0</string>
	<key>CFBundleSignature</key>
	<string>????</string>
	<key>CFBundleVersion</key>
	<string>1</string>
	<key>NSHumanReadableCopyright</key>
	<string>Copyright © 2013 Clemens Wagner. All rights reserved.</string>
	<key>NSPrincipalClass</key>
	<string></string>
</dict>
</plist>
EOF
	popd
}

make_universal() {
	outlib=$1
	shift
	params=""
	for arch in $*
	do
		params="$params -arch $arch $BUILD_DIR/$arch/$LIBNAME"
	done
	echo lipo $params -create -output $outlib
	lipo $params -create -output $outlib
}

make_framework() {
	rm -rf $FRAMEWORK_DIR
	make_bundle $BASENAME
	cp $INCLUDE_DIR/*.h $FRAMEWORK_DIR/Headers/
	make_universal $FRAMEWORK_DIR/Versions/A/$BASENAME $*
}
